/*
 Basic ESP8266 MQTT example

 This sketch demonstrates the capabilities of the pubsub library in combination
 with the ESP8266 board/library.

 It connects to an MQTT server then:
  - publishes "hello world" to the topic "outTopic" every two seconds
  - subscribes to the topic "inTopic", printing out any messages
    it receives. NB - it assumes the received payloads are strings not binary
  - If the first character of the topic "inTopic" is an 1, switch ON the ESP Led,
    else switch it off

 It will reconnect to the server if the connection is lost using a blocking
 reconnect function. See the 'mqtt_reconnect_nonblocking' example for how to
 achieve the same result without blocking the main loop.

 To install the ESP8266 board, (using Arduino 1.6.4+):
  - Add the following 3rd party board manager under "File -> Preferences -> Additional Boards Manager URLs":
       http://arduino.esp8266.com/stable/package_esp8266com_index.json
  - Open the "Tools -> Board -> Board Manager" and click install for the ESP8266"
  - Select your ESP8266 in "Tools -> Board"

*/

#include <ESP8266WiFi.h>
#include <PubSubClient.h>
#include <Wire.h>

// Update these with values suitable for your network.

const char* ssid = "ARMBIAN";
const char* password = "12345678";
const char* mqtt_server = "10.7.1.1";
const char* mqtt_id = "whoami123";


// Variables for Dissolved Oxygen
#define address 97
char computerdata[20];
byte received_from_computer = 0;
byte code = 0;
char DO_data[20];
byte in_char = 0;
int time_ = 1800;
float DO_float;
char *DO;
char *sat;
float do_float;
float sat_float;

WiFiClient espClient;
PubSubClient client(espClient);
long lastMsg = 0;
char msg[50];
int value = 0;

void setup() {
  pinMode(BUILTIN_LED, OUTPUT);     // Initialize the BUILTIN_LED pin as an output
  Serial.begin(115200);
  setup_wifi();
  client.setServer(mqtt_server, 1883);
  Wire.begin();
}

void setup_wifi() {

  delay(10);
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}


void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    // Attempt to connect
    if (client.connect(mqtt_id)) {
      Serial.println("connected");
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}
float readDO() {
  byte i = 0;
  

  Wire.beginTransmission(address);
  Wire.write('r');
  Wire.endTransmission();
  
  delay(1800);
  
  Wire.requestFrom(address, 20, 1);
  code = Wire.read();
  
  switch (code) {
    case 1:
      Serial.println("Success");
      break;
    case 2:
      Serial.println("Failed");
      break;
    case 254:
      Serial.println("Pending");
      break; //decimal 254.
    case 255:
      Serial.println("No Data");
      break; //decimal 255.
  }
  while (Wire.available()) {
    in_char = Wire.read();
    DO_data[i] = in_char;
    i += 1;
    if (in_char == 0) {
      i = 0;
      Wire.endTransmission();
      break;
    }
  }

  Serial.print("DO:");
  Serial.println(DO_data);
  DO_float = atof(DO_data);
  for (i = 0; i < 20; i++) {
      DO_data[i] = 0;
    }
  return DO_float;
}


float readpH(){
  return 7;
}

float readtemp(){
  return 0.0;
}

float readec(){
  return 0.0;
}

void loop() {

  if (!client.connected()) {
    reconnect();
  }
  client.loop();

  long now = millis();
  if (now - lastMsg > 2000) {
    lastMsg = now;
    snprintf (msg, 75, "%f", readpH());
    Serial.print("pH: ");Serial.println(msg);
    client.publish("/sensor/ph", msg);
    snprintf (msg, 75, "%f", readDO());
    Serial.print("dox: ");Serial.println(msg);
    client.publish("/sensor/dox", msg);
    snprintf (msg, 75, "%f", readtemp());
    Serial.print("temp: ");Serial.println(msg);
    client.publish("/sensor/temp", msg);
    snprintf (msg, 75, "%f", readec());
    Serial.print("EC: ");Serial.println(msg);
    client.publish("/sensor/ec", msg);
  }
}
